package com.yiring.common.swagger;

import com.github.xiaoymin.knife4j.spring.extension.OpenApiExtensionResolver;
import com.yiring.common.core.Status;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.Profile;
import org.springframework.http.HttpMethod;
import springfox.bean.validators.configuration.BeanValidatorPluginsConfiguration;
import springfox.documentation.builders.*;
import springfox.documentation.schema.ScalarType;
import springfox.documentation.service.*;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;

/**
 * Swagger Config
 *
 * @author ifzm
 * @version 0.1
 * 2019/3/11 19:13
 */

@Slf4j
@Profile(value = { "dev", "test", "preview" })
@EnableSwagger2
@Configuration
@Import(BeanValidatorPluginsConfiguration.class)
public class SwaggerConfig {

    @Value("${spring.application.name}")
    String applicationName;
    @Value("${spring.server.port}")
    String serverPort;

    @Resource
    OpenApiExtensionResolver openApiExtensionResolver;

    @Bean
    public Docket api() {
        return api(PathSelectors.any());
    }

    private Docket api(Predicate<String> paths) {
        log.info("API Doc: http://localhost:{}/doc.html", serverPort);

        return new Docket(DocumentationType.SWAGGER_2)
            .groupName("default")
            .apiInfo(apiInfo())
            .useDefaultResponseMessages(false)
            .globalRequestParameters(buildGlobalRequestParameters())
            .globalResponses(HttpMethod.GET, buildGlobalResponseMessage())
            .globalResponses(HttpMethod.POST, buildGlobalResponseMessage())
            .select()
            .apis(RequestHandlerSelectors.withClassAnnotation(Api.class))
            .paths(paths)
            .build()
            .extensions(openApiExtensionResolver.buildSettingExtensions());
    }

    private ApiInfo apiInfo() {
        String url = "https://yiring.com";

        return new ApiInfoBuilder()
            .title("Swagger 接口文档.")
            .description(applicationName)
            .version("1.0")
            .termsOfServiceUrl(url)
            .contact(new Contact("YiRing", url, "developer@yiring.com"))
            .build();
    }

    /**
     * 构建全局请求参数
     *
     * @return Token 等自定义全局参数/Header
     */
    private List<RequestParameter> buildGlobalRequestParameters() {
        List<RequestParameter> parameters = new ArrayList<>();
        parameters.add(new RequestParameterBuilder()
            .name("Authorization")
            .description("Auth Token")
            .required(true)
            .in(ParameterType.HEADER)
            .query(q -> q.model(m -> m.scalarModel(ScalarType.STRING)))
            .build());
        return parameters;
    }

    /**
     * 构建全局响应消息
     *
     * @return 所有自定义状态码消息
     */
    private List<Response> buildGlobalResponseMessage() {
        return Arrays.stream(Status.values())
            .map(status -> new ResponseBuilder()
                .code(String.valueOf(status.value()))
                .description(status.getReasonPhrase())
                .build())
            .collect(Collectors.toList());
    }

}
